@media screen and (max-width: 782px) {
  .e-form-submissions-list-table {
    /* Don't judge me... (need to override WordPress style). */ }
    .e-form-submissions-list-table.wp-list-table tr:not(.inline-edit-row):not(.no-items) > td.bulk-checkbox-column:not(.check-column) {
      width: 2.2em !important; } }

.e-form-submissions-list-table .bulk-checkbox-column {
  padding: 9px 0 0 3px;
  width: 2.2em; }

.e-form-submissions-list-table .column-actions {
  width: 11%; }
  .e-form-submissions-list-table .column-actions i {
    font-size: 15px; }

.e-form-submissions-list-table .column-id {
  width: 7%; }

.e-form-submissions-list-table .column-page {
  width: 17%;
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden; }

.e-form-submissions-list-table .column-date {
  width: 17%; }

.e-form-submissions-list-table .column-form {
  width: 20%; }

.e-form-submissions-referer-icon {
  font-size: 18px; }

.e-form-submissions-main .postbox {
  border: none; }

.e-form-submissions-main .postbox-header {
  border: 1px solid #D4D4D4;
  border-bottom: none;
  padding: 8px; }

.e-form-submissions-main__header {
  width: 100%;
  display: flex;
  justify-content: space-between;
  align-items: center; }

#e-form-submissions .eicon-success {
  color: #5cb85c; }

#e-form-submissions .eicon-error {
  color: #d9534f; }

#e-form-submissions .misc-pub-section {
  line-height: 1.3rem; }

#e-form-submissions .e-form-submissions-item-table {
  border: none;
  border-collapse: collapse; }
  #e-form-submissions .e-form-submissions-item-table td:first-child {
    width: 150px;
    background: #f7f7f7;
    font-weight: 700; }
    @media screen and (max-width: 782px) {
      #e-form-submissions .e-form-submissions-item-table td:first-child {
        /* Must set !important because wordpress set also !important. */
        width: 90px !important; } }
  #e-form-submissions .e-form-submissions-item-table td {
    border: 1px solid #D4D4D4;
    padding: 15px; }
  #e-form-submissions .e-form-submissions-item-table input:not([type=checkbox]):not([type=radio]), #e-form-submissions .e-form-submissions-item-table textarea, #e-form-submissions .e-form-submissions-item-table select {
    width: 100%;
    max-width: 400px; }

#e-form-submissions .e-export-button {
  margin-right: 15px; }
  @media screen and (max-width: 782px) {
    #e-form-submissions .e-export-button {
      display: none; } }

#e-form-submissions .tablenav .actions select {
  width: 10rem; }

#e-form-submissions .tablenav .actions .select2-container {
  float: left;
  margin-right: 6px; }
  #e-form-submissions .tablenav .actions .select2-container .select2-selection__arrow {
    height: 30px; }
  #e-form-submissions .tablenav .actions .select2-container .select2-selection--single {
    height: 30px;
    border: 1px solid #7e8993; }

input[type="search"].select2-search__field {
  line-height: 1; }

.e-form-submissions-search {
  position: relative; }
  .e-form-submissions-search__spinner {
    position: absolute;
    left: 12px;
    font-size: 11px;
    height: 100%;
    display: flex;
    align-items: center;
    color: var(--wp-admin-theme-color-darker-20, #005a87); }
  .e-form-submissions-search:focus-within .e-form-submissions-search__spinner, .e-form-submissions-search:hover .e-form-submissions-search__spinner {
    left: 30px; }

.e-form-submissions-action-log--success .e-form-submissions-action-log__message {
  background: #dff0d8;
  border-color: #5cb85c; }

.e-form-submissions-action-log--error .e-form-submissions-action-log__message {
  background: #f2dede;
  border-color: #d9534f; }

.e-form-submissions-action-log:not(:last-child) {
  border-bottom: 1px solid #D4D4D4; }

.e-form-submissions-action-log__label {
  display: inline-block;
  padding-left: 15px; }

.e-form-submissions-action-log__icon {
  display: inline-block;
  padding-left: 3px; }

.e-form-submissions-action-log__date {
  font-size: 12px;
  color: #818a91; }

.e-form-submissions-action-log__message {
  padding: 10px;
  font-size: 12px;
  color: #373a3c;
  background: #eceeef;
  border-right: 4px solid #818a91; }

.e-form-submissions-value-label:not(:last-child) {
  display: block;
  margin-bottom: 10px; }

/*# sourceMappingURL=admin-rtl.css.map */